# -*- Mode: Python; tab-width: 4 -*-
#
#	$Id: FSSession.py,v 1.1 1999/12/02 17:56:22 pavlos Exp pavlos $
#	Author: Pavlos Christoforou <pavlos@gaaros.com>
#
# ======================================================================
#
#
#  Disclaimer
#
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#     Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
#     The name of the author or GAAROS may not be used to endorse or
#     promote products derived from this software without specific prior
#     written permission
#
#
#   THIS SOFTWARE IS PROVIDED BY GAAROS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GAAROS OR HIS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
########################################################################

__version__='$Revision: 1.1 $'[11:-2]

import os,string,time
import marshal
import whrandom
import App.FindHomes
from UserDict import UserDict

import Globals

from Shared.DC.ZRDB.TM import TM
from Globals import Persistent
from Globals import HTMLFile, MessageDialog
from App.Management import Navigation, Tabs
from Acquisition import Implicit
import OFS.SimpleItem 
from OFS.Document import Document
import AccessControl.Role 
from UserDict import UserDict

FSSessionDir=os.path.join(INSTANCE_HOME,'var','FSSession')

class Store(UserDict):
    '''Implements storage object which acts like a python dictionary'''

    def __getitem__(self, key):
        # Do  not return error if key does not exist
        return self.data.get(key)

    def copy(self):
        # copy doesn't make much sense in this context
        raise AttributeError,'Copy not supported for Session storage objects'

    set=UserDict.__setitem__

    delete=UserDict.__delitem__

    def __str__(self):
        '''Pretty print contents'''
        out='<B>FSSession Contents:</B><BR>\n'
        for k,v in self.items():
            out='%s\t<LI>%s:\t%s\n'%(out,k,str(v))
        return out

class FSSession(OFS.SimpleItem.Item, Persistent, Implicit, 
		      AccessControl.Role.RoleManager,Store,TM):
    """ A File system based session management product """

    meta_type='FSSession'
    id       ='FSSession'
    title    ='FSSession'
    icon     ='misc_/FSSession/fssession'

    manage_options=(
    {'label':'Contents',   'action':'manage_main'},
    {'label':'Security',   'action':'manage_access'},
    )

    __ac_permissions__=(
    ('View management screens',
     ['manage','manage_menu','manage_main','manage_tabs' ]),
    )

    manage_main=HTMLFile('main', globals())

    def __init__(self, cookie_name):
        Store.__init__.im_func(self)
        self.cookie_name = cookie_name
        self.uid=[0] # mutable attrs treated mutably do not trigger ZODB


    def __call__(self):
        '''Initiate session'''
        self._registered=0 ## This should not have been required!!!
        self._register()
        
    def _begin(self):
        # what happens if self.load raises an exception?
        self.data.clear()
        self.load()
        
    def _abort(self):
        # since changes are only done in the tpc_finish method we can just
        # do nothing
        pass
    
    def _finish(self):
        self.dump()
        
    def load(self,tm=time,jn=os.path.join):
        
        if ((self.REQUEST.has_key('_force_new_session') and
            self.REQUEST['_force_new_session']) or not
            self.REQUEST.has_key(self.cookie_name)): # comply with SQLSession
            # create new UID
            suffix=whrandom.randint(1,100000000)
            t=tm.gmtime(tm.time())
            self.uid[0]=uid='%s%i'%(tm.strftime('%j%I%M%S',t),suffix)
            self.REQUEST[self.cookie_name]=self.uid[0]
            # self.clear() not required            
        
        else:
            self.uid[0]=uid=self.REQUEST[self.cookie_name]
            day=jn(FSSessionDir,uid[:3])
            hour=jn(day,uid[3:5])
            minute=jn(hour,uid[5:7])

            try:
                f=open(jn(minute,uid),'rb')
                self.update(marshal.load(f))
                f.close()
            except IOError:
                # create new session
                self.clear()
        
    def dump(self,tm=time,ex=os.path.exists,jn=os.path.join):

        day=jn(FSSessionDir,self.uid[0][:3])
        hour=jn(day,self.uid[0][3:5])
        minute=jn(hour,self.uid[0][5:7])
    
        if not ex(day):
            os.mkdir(day)
        if not ex(hour):
            os.mkdir(hour)
        if not ex(minute):
            os.mkdir(minute)

        # cookie expires after one year
        t=tm.gmtime(tm.time())
        t_expires=tm.strftime('%a,%d-%b-%Y 23:59:59 GMT',
                                      (t[0]+1,)+t[1:])  
        f=open(jn(minute,self.uid[0]),'wb')
        marshal.dump(self.data,f)
        f.close()
        self.REQUEST.RESPONSE.setCookie(self.cookie_name,
                                            self.uid[0],expires=t_expires)

    def url(self,url):
        if self.REQUEST.environ.has_key('HTTP_COOKIE'):
            # Client accepts cookies. Just return url. A situation can
            # arise where the client does accept cookies but there is
            # no cookie set for our domain. This method will mistake
            # the client as a non cookie enabled, but only the first
            # time.
            return url
        else:
            if '?' in url:
                return '%s&%s=%s'%(url,self.cookie_name,self.uid[0])
            else:
                return '%s?%s=%s'%(url,self.cookie_name,self.uid[0])


manage_addFSSessionForm=HTMLFile('addFSSession', globals())

def manage_addFSSession(self, cookie_name, REQUEST=None):
    """ Code from Anthony Baxter, more or less"""
    if hasattr(self.aq_base, 'FSSession'):
        return MessageDialog(
               title  ='Item Exists',
               message='This object already contains a FSSession',
               action ='%s/manage_main' % REQUEST['PARENT_URL'])
    ob=FSSession(cookie_name)
    self._setObject('FSSession', ob)
    if REQUEST:
        return self.manage_main(self,REQUEST)


    



